/* Blinker
   Copyright 2011 Stefan Schuermans <stefan@blinkenarea.org>
   Copyleft GNU public license - http://www.gnu.org/copyleft/gpl.html
   a blinkenarea.org project */

#ifndef STREAM_H
#define STREAM_H

#include <set>
#include <string>

#include <BlinkenLib/BlinkenFrame.h>

#include "StreamRecv.h"

namespace Blinker {

/// a video stream
class Stream
{
protected:
  /// set of receivers of this stream
  typedef std::set<StreamRecv *> Recvs;

public:
  /// constructor
  Stream();

  /// virtual destructor
  virtual ~Stream();

public:
  /**
   * @brief attach a stream receiver
   * @param[in] recv stream receiver to attach
   */
  void attach(StreamRecv *recv);

  /**
   * @brief detach a stream receiver
   * @param[in] recv stream receiver to detach
   */
  void detach(StreamRecv *recv);

  /**
   * @brief set current frame
   * @param[in] pFrame current frame
   */
  virtual void setFrame(stBlinkenFrame *pFrame);

  /// set current frame to none
  virtual void setNoFrame();

  /**
   * @brief get current frame
   * @param[out] pFrame current frame
   * @return if a current frame exists
   */
  bool getCurFrame(stBlinkenFrame *&pFrame);

protected:
  /// stream name
  std::string m_name;
  /// current frame (or NULL if no frame)
  stBlinkenFrame *m_pFrame;
  /// receivers of this stream
  Recvs m_recvs;

friend class StreamMgr;
}; // class Stream

} // namespace Blinker

#endif // #ifndef STREAM_H

